import logging
import sys
import xmltodict
import subprocess
from collections import namedtuple

DEBUG = True

# this is set by --run_ssh flag in check-stig-pfsense.py
RUN_SSH = True

#############################################################
# IMPORTANT CONSTANTS
#############################################################
# address of pfsense firewall that we want to STIG
firewall_ip = "192.168.1.1"
# address of private key that we are using to get into the firewall
priv_key_path = "/vagrant/.vagrant/machines/default/virtualbox/private_key"

# data types
#  Finding: basically an enum{UNIMPLEMENTED, PASS, FAIL}
#  ProcessOut: basically a class with two fields, stdout and stderr
UNIMPLEMENTED = 1
PASS = 2
FAIL = 3

class Finding():
    def __init__(self, finding, msg):
        if (finding == UNIMPLEMENTED or
            finding == PASS or
            finding == FAIL):
            self.finding = finding
            self.msg = msg
        else:
            logging.critical("finding [{}] does not exist".format(finding))
            sys.exit()

    def finding_to_str(self):
        if self.finding == UNIMPLEMENTED:
            return "Unimplemented Rule"
        elif self.finding == PASS:
            return "NotAFinding"
        elif self.finding == FAIL:
            return "Open"

    def __str__(self):
        return "{}: msg: [{}]".format(self.finding_to_str(),
                               self.msg)

ProcessOut = namedtuple("ProcessOut", ['stdout', 'stderr'])

#############################################################
# helper methods
#############################################################
def execute_on_box(cmd):
    ssh_part = ["ssh", "-i", priv_key_path, "root@{}".format(firewall_ip)]
    ssh_cmd = ssh_part + [cmd]
    if DEBUG:
        print("SSH command is {}".format(ssh_cmd))
    if RUN_SSH:
        if DEBUG:
            print("Running . . .")
        p = subprocess.Popen(ssh_cmd,
                            stderr=subprocess.PIPE,
                            stdout=subprocess.PIPE)
        p.wait()
        return ProcessOut(p.stdout.read().decode().strip(),
                          p.stderr.read().decode().strip())
    else:
        return ProcessOut("", "")

# returns the xml config file as a python dict
def parse_config():
    # config.xml is located at /cf/conf/config.xml
    CONFIG_XML = "/cf/conf/config.xml"

    return xmltodict(open(CONFIG_XML))

#############################################################
# exported methods
#############################################################
def validate_findings(finding_list):
    is_pass = lambda f: f == PASS
    return (PASS if all(map(is_pass, finding_list))
            else FAIL)

def check_artifact(cmd, artifact,
                   no_artifact=False):
    out = execute_on_box(cmd)
    # if DEBUG:
        # print("out.stdout {}\nartifact {}".format(out.stdout, artifact))
    # this is the case where we do not have a clear
    # artifact but we still want to show the output.
    # For example, rule, V-3062 which shows that passwords are not
    # stored in the clear
    out_with_context = ProcessOut("root$ {}\n".format(cmd) + out.stdout,
                                  out.stderr)
    if no_artifact:
        return (PASS, out_with_context)
    if out.stdout == artifact:
        return (PASS, out_with_context)
    else:
        return (FAIL, out_with_context)

