#!/usr/bin/python3
USAGE = "{} <xmlfile>"

import sys
import xmltodict
from collections import namedtuple

#####################################
# data types
#####################################
Row = namedtuple("Row", ['id', 'short_title', 'title', 'description', 'fix', 'check', 'severity'])

#####################################
# helper methods
#####################################
def sanitize_quote(u_str):
    return u_str.replace("\"", "\'")

def quote(s):
    return "\"{}\"".format(s)

# takes in a dictionary form of stig XML file
# and creates a list of Row
def create_rowlist(xml_dict):
    # expects dict with keys
    # ['@id', 'title', 'description', 'Rule']
    def create_row(item):
        v_id = item["@id"]
        short_title = item["title"]
        title = item["Rule"]["title"]
        # this one is special cause xmltodict does not properly
        # handle the <VulnDiscussion> tags
        description = item["Rule"]["description"]
        start = description.find("<VulnDiscussion>") + len("<VulnDiscussion>")
        end = description.find("</VulnDiscussion>")
        description = description[start:end]

        fixes = item["Rule"]["fixtext"]["#text"]
        check = item["Rule"]["check"]["check-content"]
        severity = item["Rule"]["@severity"]
        return Row(*map(sanitize_quote, [v_id, short_title,
            title, description,
            fixes, check, severity]))

    return map(create_row, xml_dict["Benchmark"]["Group"])

#####################################
# Ouput methods - just CSV for now
#####################################

# takes in a List of Row
# prints properly formatted .csv to stdout
def csvify(rows):
    print("\t".join(['rule', 'title', 'fixtext']))
    for row in rows:
        line = [row.id, row.title, row.fix]
        # santize unicode characters
        # and surround with quotes
        line = map(quote, line)
        print("\t".join(line))


if __name__ == "__main__":
    if len(sys.argv) != 2:
        print(USAGE.format(sys.argv[0]))
        sys.exit(1)
    xml_file = sys.argv[1]

    xml_dict = xmltodict.parse(open(xml_file).read())
    l = create_rowlist(xml_dict)
    csvify(l)


